# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Notifications is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Builderes for notifications."""

from abc import ABC, abstractmethod


class NotificationBuilder(ABC):
    """Base notification builder."""

    context = []
    """List of ContextGenerator to update notification context."""

    recipients = []
    """List of RecipientGenerator."""

    recipient_filters = []
    """List of RecipientFilter."""

    recipient_backends = []
    """List of RecipientBackendGenerator."""

    type = "Notification"

    @classmethod
    @abstractmethod
    def build(cls, **kwargs):
        """Build notification based on type and additional context."""
        raise NotImplementedError()

    @classmethod
    def resolve_context(cls, notification):
        """Resolve all references in the notification context."""
        for ctx_func in cls.context:
            # NOTE: We assume that the notification is mutable and modified in-place
            ctx_func(notification)
        return notification

    @classmethod
    def build_recipients(cls, notification):
        """Return a dictionary of unique recipients for the notification."""
        recipients = {}
        for recipient_func in cls.recipients:
            recipient_func(notification, recipients)
        return recipients

    @classmethod
    def filter_recipients(cls, notification, recipients):
        """Apply filters to the recipients."""
        for recipient_filter_func in cls.recipient_filters:
            recipient_filter_func(notification, recipients)
        return recipients

    @classmethod
    def build_recipient_backends(cls, notification, recipient):
        """Return the backends for recipient."""
        backends = []
        for recipient_backend_func in cls.recipient_backends:
            recipient_backend_func(notification, recipient, backends)
        return backends



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 CERN.
# Copyright (C) 2023-2024 Graz University of Technology.
#
# Invenio-Notifications is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio module for notifications support."""

from .ext import InvenioNotifications
from .proxies import current_notifications, current_notifications_manager

__version__ = "1.2.0"

__all__ = (
    "__version__",
    "current_notifications",
    "current_notifications_manager",
    "InvenioNotifications",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Graz University of Technology.
#
# Invenio-Notifications is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Filters for notification recipients."""

from abc import ABC, abstractmethod

from invenio_records.dictutils import dict_lookup


class RecipientFilter(ABC):
    """Recipient filter for a notification."""

    @abstractmethod
    def __call__(self, notification, recipients):
        """Filter recipients."""
        raise NotImplementedError()


class KeyRecipientFilter(RecipientFilter):
    """Recipient filter based on a given key."""

    def __init__(self, key):
        """Initialize with key for lookup."""
        super().__init__()
        self._key = key

    def __call__(self, notification, recipients):
        """Filter recipients."""
        recipient_key = dict_lookup(notification.context, self._key)
        if not isinstance(recipient_key, str):
            return recipients

        if recipient_key in recipients:
            del recipients[recipient_key]

        return recipients
